#include "stdafx.h"
#include <stdlib.h>
#include <stdio.h>

#include "resource.h"

#include "os.h"
#include "transform.h"
#include "xprocess.h"
#include "sprite.h"
#include "wimp.h"

#include "wos.h"

#include "poll.h"
#include "pane.h"
#include "temp.h"
#include "trans.h"
#include "deb.h"
#include "err.h"
#include "crc.h"
#include "osreg.h"
#include "werr.h"
#include "wimpt.h"
#include "bits.h"


#include "etc.h"


#include "appreg.h"







// <reg version> - <product code> - <user number> - <crc>
//      XX       - XXXX           - XXXXXX        - XXXXXX
//      2          4                6               8

// HKLM/Software/DavidPilling/<programname>/RegData - installation time
//                                                  - key + name


#define REGVN     2
#define PRODUCTN  4
#define USERN     6
#define CRCN      8

#define NAMEMAX   32
#define KEYMAX    256
#define TIMEMAX   256


static os_error * appregwindow2(wimp_w parent,int tedious);


static int * appproductcodes;



static int regcrc(TCHAR * name,int regversion,int productcode,int usernumber)
{
 int   crc;
 int   len;
 TCHAR temp[256];
 int   i;

 crc=-1;
 len=ustrlen(name);
	if(len>sizeof(temp)/sizeof(TCHAR)) len=sizeof(temp)/sizeof(TCHAR);

 for(i=0;i<len;i++)
 {
  temp[i]=name[len-i];
 }

 crcblock32(&crc,temp,len);
 crcblock32(&crc,(char*)&regversion,sizeof(regversion));
 crcblock32(&crc,(char*)&productcode,sizeof(productcode));
 crcblock32(&crc,(char*)&usernumber,sizeof(usernumber));


// cprintf("name %s ver %x prod %x user %x crc %x",name,regversion,productcode,usernumber,crc);

 return(crc);
}






char * appregmakekey(TCHAR * name,int regversion,int productcode,int usernumber)
{
 static char regkey[REGVN+1+PRODUCTN+1+USERN+1+CRCN+1];

#ifdef _DEBUG

 int         crc;

 crc=regcrc(name,regversion,productcode,usernumber);

 xsprintf(sizeof(regkey)/sizeof(TCHAR),NULL,regkey,"%x-%x-%x-%x",regversion,productcode,usernumber,crc);

#endif

 return(regkey);
}



static char * readhex(TCHAR * string,int * value)
{
 int check;
 int ch;

 check=0;

 while(1)
 {
  ch=*string;
  ch=toupper(ch);

  if(ch>='0' && ch<='9') check=check*16+(ch-'0');
  else
  if(ch>='A' && ch<='F') check=check*16+(ch-'A'+10);
  else
  if(ch==' ' || ch=='\t')
  {

  }
  else                   break;

  string++;
 }

 *value=check;

 return(string);
}



// take key string 00-0000 etc. and split into values
// <reg version> - <product code> - <user number> - <crc>

static os_error * appregexpkey(TCHAR * key,int * regversion,int * productcode,int * usernumber,int * crc)
{
 TCHAR    * p;
 int        value;


 p=key;

 p=readhex(p,&value);
 if(*p++=='-')
 {
  *regversion=value;
  p=readhex(p,&value);
  if(*p++=='-')
  {
   *productcode=value;
   p=readhex(p,&value);
   if(*p++=='-')
   {
    *usernumber=value;
    p=readhex(p,&value);
    if(*p==0)
    {
     *crc=value;
     return(NULL);
    }
   }
  }
 }

 return(geterror(EBADREG));
}



static os_error * appregstring(TCHAR * appname,TCHAR * string,int * size,int write)
{
 os_error * err;
 TCHAR      keypath[256];

 xsprintf(NITEM(keypath),NULL,keypath,TEXT("HKEY_LOCAL_MACHINE\\Software\\DavidPilling\\%s\\RegData"),appname);

 if(write) err=regwritestr(keypath,string);
 else      err=regreadstr(keypath,string,size);

 return(err);
}



static void appregcodestring(TCHAR * string)
{
 TCHAR * p;
 int     ch;

 p=string;

 cprintf("code %s",string);

 while((ch=*p)!=0)
 {
  ch=ch+(256-32)/2;
  if(ch>255) ch-=(256-32);
  *p++=(char)ch;
 }

 cprintf("code %s",string);
}





// write key+name string to registry

static os_error * appregwritekey(TCHAR * appname,TCHAR * name,TCHAR * key)
{
	os_error * err;
 TCHAR      temp[256];
 int        size;

 xsprintf(NITEM(temp),NULL,temp,TEXT("\\N%s\\K%s"),name,key);

 appregcodestring(temp);

 size=sizeof(temp)/sizeof(TCHAR);
 err=appregstring(appname,temp,&size,1);

 return(err);
}





// read key+name string from registry


static os_error * appregreadkey(TCHAR * appname,TCHAR * name,TCHAR * key,TCHAR * time)
{
 TCHAR   temp[256];
 int     size;
 TCHAR * p;
 TCHAR * d;
 int     ch;

 *name=0;
 *key=0;
 *time=0;

 size=sizeof(temp)/sizeof(TCHAR);
 appregstring(appname,temp,&size,0);

 cprintf("appregstring %s",temp);

 appregcodestring(temp);

 p=temp;
 while((ch=*p++)!=0)
 {
  if(ch=='\\')
  {
   ch=*p++;
   if(ch=='T') d=time;
   else
   if(ch=='K') d=key;
   else
   if(ch=='N') d=name;
   else
   {
    break;
   }

   while((ch=*p)!=0)
   {
    if(ch=='\\') break;
    *d++=(char)ch;
    p++;
   }

   *d=0;
  }
 }


 return(NULL);
}





static os_error * checkkey(TCHAR * name,TCHAR * key)
{
 os_error * err;
 int        crc;
 int        check;
 int        regversion;
 int        productcode;
 int        usernumber;
 int        code;
 int        i;


 err=appregexpkey(key,&regversion,&productcode,&usernumber,&check);
 if(!err)
 {
  crc=regcrc(name,regversion,productcode,usernumber);
  cprintf("name=%s ver %x prod %x user %x crc %x check %x",name,regversion,productcode,usernumber,crc,check);

  if(crc!=check) err=geterror(EBADREG);
  else
  {
   i=0;
   while(1)
   {
    code=appproductcodes[i];
    if(code==APPREGLASTCODE)
    { 
     err=geterror(EBADREG);
     break;
    }
    if(productcode==code) break;
    i++;
   }
  }
 }

 return(err);
}



/*******************************************************************************/
// this is the window that says the evaluation time has run out



static os_error * appregtedicon(wimp_w handle,uservalue userhandle,wimp_mousestr * m);
static os_error * appregtedkey(wimp_w handle,uservalue userhandle,int icon,int * key);
static os_error * appregtedclosew(wimp_w appregtedhandle,uservalue parent);

//static int appregtedhandle;
//static int appregtedparent;

static wimp_w   appreghandle;
static int      appregfromtedious;
static appregfn appregevent;


static os_error * appregtedcloselo(wimp_w appregtedhandle,wimp_w parent,int flags)
{
 os_error * err;

 if(flags==0)
 {
  err=NULL;
  pollquit(0);
 }
 else
 {
  err=enablewindow(parent,1,1);

  if(!err)
  {
   remclickevent(appregtedicon,appregtedhandle,(uservalue)parent);
   remcloseevent(appregtedclosew,appregtedhandle,(uservalue)parent);
   remkeyevent(appregtedkey,appregtedhandle,(uservalue)parent);

   err=closedown(&appregtedhandle);
  }
 }


 return(err);

 USE(flags);
}


static os_error * appregtedclosew(wimp_w appregtedhandle,uservalue parent)
{
 return(appregtedcloselo(appregtedhandle,(wimp_w)parent,0));
}



static void appregtedzero(uservalue appregtedhandle)
{

 if(appreghandle) // if they're entering key, keep open
 {
  addscheduledevent(appregtedzero,(uservalue)appregtedhandle,1500);
 }
 else
 {
  wimp_w parent;

  getparent((wimp_w)appregtedhandle,&parent);
  appregtedcloselo((wimp_w)appregtedhandle,parent,1);
 }
}


// passed parent handle of register window, might be tedious window, might be app window

static os_error * appregkeyok(wimp_w appregtedhandle)
{
 os_error * err;

 err=NULL;
 
 if(appregtedhandle)
 {
  wimp_w parent;

  getparent(appregtedhandle,&parent);
  remscheduledevent(appregtedzero,(uservalue)appregtedhandle);
  appregtedcloselo(appregtedhandle,parent,1);
 }

 return(err);
}




static os_error * appregtedkey(wimp_w handle,uservalue userhandle,int icon,int * key)
{
 os_error * err;

 err=NULL;
 
 if(*key==27 || *key==13)
 {
  *key=-1;
 }



 return(err);

 USE(icon);
 USE(userhandle);
 USE(handle);
}



static os_error * appregtedicon(wimp_w handle,uservalue userhandle,wimp_mousestr * m)
{
 os_error * err;
 char       v[256];


 err=NULL;

 switch(m->i)
 {
  case IDC_TED_ENT:
                   err=appregwindow2(handle,1);
                   break;


  case IDC_TED_REG:
                   tsprintf(sizeof(v)/sizeof(TCHAR),v,TEXT("{WEB}"));
                   ShellExecute((HWND)handle,TEXT("open"),v,NULL,NULL,0); 
                   break;

  case IDC_TED_DOC:
                   if(appregevent) err=appregevent(handle,APPREGDOC);
                   break;

 }

 return(err);

 USE(userhandle);
}



static os_error * dotedium(int diff,wimp_w parent)
{
 os_error * err;
 int        delay=6000;
 wimp_w     appregtedhandle;

 err=NULL;

 if(diff<0 || diff>60*24) delay=12000;
 else
 if(diff>45*24)           delay=6000;
 else
 if(diff>37*24)           delay=3000;
 else                     delay=1500;

// appregtedparent=parent;

 if(!err)
 {
  enablewindow(parent,0,1);

  err=createwindowtitleparent(TTEDIOUS,TEXT("{REGNOW}"),&appregtedhandle,parent);

  if(!err)
  {
   addclickevent(appregtedicon,appregtedhandle,(uservalue)parent);
   addcloseevent(appregtedclosew,appregtedhandle,(uservalue)parent);
   addkeyevent(appregtedkey,appregtedhandle,(uservalue)parent);
   addscheduledevent(appregtedzero,(uservalue)appregtedhandle,delay);

   popup(appregtedhandle,0);

   setfocus(appregtedhandle);
  }
 }

 return(err);
}


os_error * appreg2(TCHAR * appname,wimp_w parent,int flags)
{
 os_error    * err;
 char          name[NAMEMAX];
 char          key[KEYMAX];
 char          time[TIMEMAX];
 LARGE_INTEGER ftime;
 int           diff;

 err=appregreadkey(appname,name,key,time);
 if(!err)
 {
  if(*name && *key)
  {
   err=checkkey(name,key);
  }
  else
  if(flags & APPREGABS)
  {
   err=geterror(EBADREG);
  }
  else
  if(*time)
  {
   diff=atoi(time); // install time

   GetSystemTimeAsFileTime((FILETIME*)&ftime);

   ftime.QuadPart/=10000000;   // seconds from 100 nanosecs
   ftime.QuadPart/=3600;       // hours

   diff=ftime.LowPart-diff;    // now - install time
   if(diff<0 || diff>30*24)
   {
    err=dotedium(diff,parent);
   }
  }
  else
  {
   err=geterror(EBADREG);
  }
 }
 else
 {
  err=geterror(EBADREG);
 }

 return(err);
}



os_error * appreg(wimp_w parent)
{
 return(appreg2(wimpt_programname(),parent,0));
}



// do tedious window after the fixed time is past

os_error * appregfixed(wimp_w parent,int flags,int year,int month,int day)
{
 os_error     * err;
 SYSTEMTIME     systemt;
 LARGE_INTEGER  ftime;
 int            diff;

 err=NULL;

 memset(&systemt,0,sizeof(systemt));

 systemt.wYear=(unsigned short)year;
 systemt.wMonth=(unsigned short)month;
 systemt.wDay=(unsigned short)day;

 SystemTimeToFileTime(&systemt,(FILETIME*)&ftime);


 ftime.QuadPart/=10000000;   // seconds from 100 nanosecs
 ftime.QuadPart/=3600;       // hours

 diff=ftime.LowPart;

 GetSystemTimeAsFileTime((FILETIME*)&ftime);

 ftime.QuadPart/=10000000;   // seconds from 100 nanosecs
 ftime.QuadPart/=3600;       // hours

 diff=ftime.LowPart-diff;    // now - fixed time

 if(diff>0) err=dotedium(diff,parent);

 return(err);

 USE(flags);
}



/*****************************************************************************/
// this is the window that accepts the key



static os_error * appregicon(wimp_w handle,uservalue userhandle,wimp_mousestr * m);
static os_error * appregkey(wimp_w handle,uservalue userhandle,int icon,int * key);
static os_error * appregclosew(wimp_w w,uservalue parent);



static os_error * appregcloselo(wimp_w w,wimp_w parent,int ok)
{
 os_error * err;
 char       name[NAMEMAX];
 char       key[KEYMAX];
 int        keyok=0;

 err=NULL;

 if(ok)
 {
           err=readicon(w,IDC_REG_USER,name,sizeof(name)/sizeof(TCHAR));
  if(!err) err=readicon(w,IDC_REG_KEY,key,sizeof(key)/sizeof(TCHAR));
  if(!err)
  {
   err=checkkey(name,key);
   if(!err)
   {
    err=appregwritekey(wimpt_programname(),name,key);
    if(!err)
    {
     messagebox(TEXT("{PROK}"));
     keyok=1;
    }
   }
  }
 }

 if(!err)
 {
  remclickevent(appregicon,appreghandle,(uservalue)parent);
  remcloseevent(appregclosew,appreghandle,(uservalue)parent);
  remkeyevent(appregkey,appreghandle,(uservalue)parent);

  err=closedown(&appreghandle);

  if(!err && keyok && appregfromtedious) err=appregkeyok(parent);
 }

 return(err);

}

static os_error * appregclosew(wimp_w w,uservalue parent)
{
 return(appregcloselo(w,(wimp_w)parent,0));
}


static os_error * appregkey(wimp_w handle,uservalue parent,int icon,int * key)
{
 os_error * err;

 err=NULL;
 
 if(*key==27)
 {
  err=appregcloselo(handle,(wimp_w)parent,0);
  *key=-1;
 }

 return(err);

 USE(icon);
}



static os_error * appregicon(wimp_w handle,uservalue parent,wimp_mousestr * m)
{
 os_error * err;

 err=NULL;

 switch(m->i)
 {
               case     IDOK:
               case IDCANCEL:
                             err=appregcloselo(handle,(wimp_w)parent,m->i==IDOK);
                             break;

 }

 return(err);
}





static os_error * appregwindow2(wimp_w parent,int tedious)
{
 os_error * err;
 TCHAR      name[NAMEMAX];
 TCHAR      key[KEYMAX];
 TCHAR      time[TIMEMAX];


 if(appreghandle) err=forward(appreghandle,0,NULL);
 else
 {
  err=createwindowtitleparent(TREGNOW,TEXT("{REGNOW}"),&appreghandle,parent);

  if(!err)
  {
   err=writeiconf(appreghandle,IDC_REG_NAME,TEXT("{NAME}"));
			if(!err)
			{
    err=appregreadkey(wimpt_programname(),name,key,time);
 			if(!err)
 			{
 		  if(*name && *key)
     {
      err=checkkey(name,key);
     }
					else
					{
						err=geterror(EBADREG);
					}
 			}



 			if(err) err=writeiconf(appreghandle,IDC_REG_STATUS,TEXT("{REGUN}"));
 			else
  		{
   		err=writeiconf(appreghandle,IDC_REG_STATUS,TEXT("{REGOK}"));
     if(!err) err=writeicon(appreghandle,IDC_REG_USER,name);
 			}

    addclickevent(appregicon,appreghandle,(uservalue)parent);
    addcloseevent(appregclosew,appreghandle,(uservalue)parent);
    addkeyevent(appregkey,appreghandle,(uservalue)parent);

    popup(appreghandle,0);

    iecarrot(appreghandle,IDC_REG_USER);
			}
			appregfromtedious=tedious;
  }
 }

 return(err);
}


os_error * appregwindow(wimp_w parent)
{
 return(appregwindow2(parent,0));
}



static templates temptable[]=
{
 TREGNOW,1,0,"RegNow",
 TTEDIOUS,1,0,"Tedious",
 TLAST,  0,0,""
};


static templink templatelink;






os_error * appreginit(appregfn xappregevent,int * xproductcodes)
{

 appregevent=xappregevent;
 appproductcodes=xproductcodes;

 return(addtemplates(NULL,NULL,temptable,&templatelink));
}

